#!/usr/bin/perl
#
# Copyright (c) 2004 Mathieu Roy <yeupou at gna org>
# http://yeupou.coleumes.org
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA
#
# $Id: integcheck-compare.pl,v 1.3 2005/01/15 08:28:11 yeupou Exp $
#
# Part of integcheck, http://gna.org/projects/integcheck
#
# This script compares two output of integcheck an print changed files.
# It will work with previously downloaded files. It permits to re-run the
# test with using the network.

use strict;
use Getopt::Long;


# Get command line input.
my ($getopt, $help, $current, $previous, $verbose);
my $limit = 1000;

eval {
    $getopt = GetOptions("current-list=s" => \$current,
			  "previous-list=s" => \$previous,
			  "limit=s" => \$limit,
			  "verbose" => \$verbose,
			 "help" => \$help);
};

if ($help) {
    print "Usage: $0 [OPTIONS]\n\n";
    print "  -h, --help                  display this help and exit\n";
    print "      --current-list=</path>  list that contain the currently md5sums\n";
    print "      --previous-list=</path> list that contain the previous md5sums\n";
    print "      --limit=<number>        maximum count of changed files (default 1000)";
    print "\n";
    exit;
}

sub IntegCheck_checkinput {
    die "You forgot to give the current-list argument or previous-list argument. Exiting" unless $_[0];
    die "File \"$_[0]\" not found. Exiting" unless -e $_[0];
    die "File \"$_[0]\" not readable. Exiting" unless -r $_[0];

}

IntegCheck_checkinput($current);
IntegCheck_checkinput($previous);


# Create an hash with the previous results
my %previous;
my $previous_count;

open(PREVIOUS, "< $previous");
while(<PREVIOUS>)
{
    next unless(/^\#\#\#\# Start HERE \#\#\#\#/ .. /^\#\#\#\# Finish HERE \#\#\#\#/);
    next if /^\#/;
    my ($file, $hash) = split(":", $_);
    chomp($hash);
    $previous{$file} = $hash;
    $previous_count++;
}
close(PREVIOUS);

# Create an hash with the current results
my %current;
my $current_count;

open(CURRENT, "< $current");
while(<CURRENT>)
{
    
    next unless(/^\#\#\#\# Start HERE \#\#\#\#/ .. /^\#\#\#\# Finish HERE \#\#\#\#/);
    next if /^\#/;
    my ($file, $hash) = split(":", $_);
    chomp($hash);
    $current{$file} = $hash;
    $current_count++;
}
close(CURRENT);


# Compare, take the current file as basis:
#         The drawback is the fact that we'll miss the missing files (unless
#         we re-run a serie of tests), but it's far more important to know
#         which files has been added.
my $changed;
my $added;

while(my ($file, $hash) = each(%current)) {
    unless (exists($previous{$file})) {
	# file has been added 
	if ($added < $limit || $verbose) {
	    print "$file is new!\n";
	} elsif ($added eq $limit) { 
	    print "\nMore than 1000 files have been added!!!\n";
	    print "We stop the listing now. If you want the full list,\n";
	    print "run integcheck-compare.pl with --verbose mode\n";
	} else {
	    print "." ;
	}
	$added++;
    } else {
	# entry exists
	if ($previous{$file} eq $hash) {
	    print "$file is unchanged.\n" if $verbose;
	} else {
	    if ($changed < $limit || $verbose) {
		print "$file changed!\n";
	    } elsif ($changed eq $limit) {
		print "\nMore than $limit files have changed!!!\n";
		print "We stop the listing now. If you want the full list,\n";
		print "run integcheck-compare.pl with --verbose mode\n";
	    } else {
		# after the limit, show each changed file by a dot.
		print "."
		}
	    $changed++;
	}
    }
}



# Print final warning
print "\nWarning:\n\tThe file count does not match, there were $previous_count files previously\n\tand there is $current_count files now.\n" if $previous_count ne $current_count;
print "\nWarning:\n\t$changed files were changed. Take action ASAP if necessary.\n" if $changed;
print "\nWarning:\n\t$added files are new. Take action ASAP if necessary.\n" if $added;

print "\nWarning:\n\tNo files have been found in the current md5 list. There is a problem, Please check.\n" unless $current_count;
print "\nWarning:\n\tNo files have been found in the previous md5 list. There is a problem, Please check.\n" unless $previous_count;

# EOF
